% compare AP and COPAP on mpeg7 silhouette database

close all;
clear all;

% you'll need to tweak the next one or two lines
cd('C:\Documents and Settings\cscott\Desktop\COPAP\mpeg7');
dirinfo=dir('./mpeg7data');
dirinfo=dirinfo(3:end); % omit '.' and '..'
% code below assumes files are sorted alphabetically with all capitalized names coming
% before any uncapitalized name

L=length(dirinfo);

% generate shape contexts. just need to run once.
if 0
    numPoints=100;
    mpeg7scdata=cell(L);
    mpeg7scdataFlip=cell(L);
    for i=1:70
        %figure(1)
        for j=1:20
            %subplot(4,10,j)
            x = double(imread(strcat('./mpeg7data/',dirinfo(20*(i-1)+j).name)));
            if i==3
                if j==4
                    x=~x;   % enables proper boundary extraction for this image, which appears inverted with respect to others in the same class
                end
            end
            %imagesc(x);
            %colormap gray  
            %subplot(4,10,j+20)     % uncomment lines 132-3 in get_sc.m to display contours
            mpeg7scdata{20*(i-1)+j}=get_sc(x,numPoints);
            xFlip=fliplr(x);
            mpeg7scdataFlip{20*(i-1)+j}=get_sc(xFlip,numPoints);
            %colormap gray
        end
        %pause
    end
    
    save mpeg7scdata100 mpeg7scdata mpeg7scdataFlip
    
end

load mpeg7scdata100

tmp=ones(20,1)*(1:70);
classList=tmp(:);

% vector of dummy costs to try out. more efficient to do several at once.
dumCost=[.8];
nDum=length(dumCost);

scores=zeros(L,L,nDum);
scoresAP=zeros(L,L,nDum);
kOpt=zeros(L,L,nDum);
%load mpeg7scores100

startTime=cputime;
lastSave=clock;
for i=1:L
    tic
    disp(['i = ' num2str(i)])
    BH1=mpeg7scdata{i};
    BH1Flip=mpeg7scdataFlip{i};
    for j=i+1:L
        costmat=get_costmat(BH1,mpeg7scdata{j});
        costmatFlip=get_costmat(BH1Flip,mpeg7scdata{j});
        m=size(costmat,1);
        n=size(costmat,2);
        N=m+n;
        for d=1:nDum
            % cyclic order preserving assignment
            [P,score,sOpt,k]=copap3(costmat,dumCost(d));
            [P,scoreFlip,sOpt,kFlip]=copap3(costmatFlip,dumCost(d));
            kOpt(i,j,d)=k;
            if score <= scoreFlip
                scores(i,j,d)=score;
                kOpt(i,j,d)=k;
            else
                scores(i,j,d)=scoreFlip;
                kOpt(i,j,d)=kFlip;
            end
            
            % unordered assignment
            costmat2=ones(N,N)*dumCost(d);
            costmat2(1:m,1:n)=costmat;
            [P,score]=jv(costmat2);
            costmat2(1:m,1:n)=costmatFlip;
            [P,scoreFlip]=jv(costmat2);
            if score <= scoreFlip
                scoresAP(i,j,d)=score;
            else
                scoresAP(i,j,d)=scoreFlip;
            end
            
        end
    end
    toc
    if etime(clock,lastSave) > 3600 % save data once every hour
        save mpeg7scores100 scores scoresAP kOpt
        lastSave=clock;
    end
end
totalTime=cputime-startTime

for d=1:nDum
    scores(:,:,d) = scores(:,:,d) + scores(:,:,d)';
    scoresAP(:,:,d) = scoresAP(:,:,d) + scoresAP(:,:,d)';
end

save mpeg7scores100 scores scoresAP kOpt

positionCount=zeros(40,nDum);
positionCountAP=zeros(40,nDum);
for d=1:nDum
    % cyclic
    for i=1:70
        for j=1:20
            [topscores, ind]=sort(scores(20*(i-1)+j,:,d));
            tmp = (i==classList(ind(1:40)));
            RecRate(20*(i-1)+j,d) = sum(tmp)/20;
            positionCount(:,d)=positionCount(:,d)+tmp;
        end
        classAvgRecRate(i,d)=mean(RecRate(20*(i-1)+1:20*(i-1)+20,d));
    end
    avgRecRate(d) = mean(RecRate(:,d));
    positionRate(:,d)=positionCount(:,d)/1400;

    % unconstrained matching
    for i=1:70
        for j=1:20
            [topscores, ind]=sort(scoresAP(20*(i-1)+j,:,d));
            tmp = (i==classList(ind(1:40)));
            RecRateAP(20*(i-1)+j,d) = sum(tmp)/20;
            positionCountAP(:,d)=positionCountAP(:,d)+tmp;
        end
        classAvgRecRateAP(i,d)=mean(RecRateAP(20*(i-1)+1:20*(i-1)+20,d));
    end
    avgRecRateAP(d) = mean(RecRateAP(:,d));
    positionRateAP(:,d)=positionCountAP(:,d)/1400;

end
    
close all
for d=1:nDum
    figure(d)
    fs=20;
    axes('FontSize',fs);
    plot(positionRate(1:20,d))
    hold on
    plot(positionRateAP(1:20,d),':');
    legend('COPAP','AP','Location','SouthWest')
    ylabel('Recognition rate','FontSize',fs)
    xlabel('Rank','FontSize',fs)
end